using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Mvc.Ajax;
using EPiServer.Shell.Gadgets;
using EPiServer.Shell.Gadgets.Contracts;
using EPiServer.Shell.Web;
using EPiServer.Shell;
using BearWare.PixlrGadget.Models;
using System.Globalization;
using EPiServer.Core;

namespace BearWare.PixlrGadget.Controllers
{
    [Gadget(Name = "Pixlr Image Editor", ClientScriptInitMethod = "pixlr.init")]
    [ScriptResource("~/Content/PixlrGadget/Pixlr.js")]
    [CssResource("~/Content/PixlrGadget/Pixlr.css")]
    public class GadgetController : Controller, IGadgetNotify
    {
        /// <summary>
        /// A default constructor is needed by the framework.
        /// </summary>
        public GadgetController() { }

        [GadgetAction(ActionType.Menu, ActionName = "Index", TextResourceKey = "SelectImage", ResourceType = typeof(BearWare.PixlrGadget.Views.Gadget.GadgetResources))]
        public ActionResult Index(Guid gadgetId)
        {
            ViewModel model = GetIndexState(gadgetId);

            if (!string.IsNullOrEmpty(model.FileInfo.SavedFilePath))
            {
                string format = Translate("/BearWare/PixlrPlugin/SavedImageFormatString");
                string imgLink = string.Format("<a href=\"{0}\">{1}</a>", model.FileInfo.SavedFilePath, model.FileInfo.SavedFilePath);
                model.SavedImageMessage = string.Format(format, imgLink);
            }

            return View(model);
        }

        public ActionResult Edit(Guid gadgetId, [Bind(Exclude = "gadgetId")] ViewModel model)
        {
            string targetFolder = string.Empty;
            if (!string.IsNullOrEmpty(model.FileInfo.TargetFolder))
                targetFolder = "&targetfolder=" + Util.UrlEncode(model.FileInfo.TargetFolder);

            string folderPath = Util.FolderPath(model.FileInfo.FilePath);

            string targeturlformat = "{0}/Content/PixlrGadget/GIR.aspx?guid={1}&forcenew={2}&folderpath={3}&versioningsupport={4}&gadgetid={5}&orgTitle={6}&epslanguage={7}{8}";
            string targeturl = Util.UrlEncode(string.Format(targeturlformat, 
                                                            SiteUrl,
                                                            Util.FileGuid(model.FileInfo.FilePath),
                                                            model.FileInfo.SaveAsNew.ToString(),
                                                            folderPath,
                                                            Util.VersioningSupport(model.FileInfo.FilePath),
                                                            gadgetId.ToString(),
                                                            Util.FileName(model.FileInfo.FilePath),
                                                            Locale,
                                                            targetFolder)
                                );

            string image = string.Empty;
            if (!string.IsNullOrEmpty(model.FileInfo.FilePath))
            {
                image = "&title=" + Util.FileName(model.FileInfo.FilePath);
                image += "&image=" + Util.UrlEncode(SiteUrl + model.FileInfo.FilePath);
            }

            string targetInfo;
            if (!string.IsNullOrEmpty(model.FileInfo.TargetFolder))
                targetInfo = model.FileInfo.TargetFolder;
            else if (!string.IsNullOrEmpty(folderPath))
                targetInfo = folderPath;
            else
                targetInfo = LanguageManager.Instance.TranslateFallback("/BearWare/PixlrPlugin/DefaultFolder/Short", "Default folder");
            targetInfo = string.Format(" ({0})", targetInfo);

            string urlformat = "http://www.pixlr.com/editor/?wmode=transparent&loc={0}&target={1}&referrer={2}{3}";
            string url = string.Format(urlformat,
                                        Locale,
                                        targeturl,
                                        Util.UrlEncode(SiteUrl + targetInfo),
                                        image);

            FileInfo postedData = model.FileInfo;
            model = Data.LoadViewModel(gadgetId);
            model.FileInfo = postedData;
            model.EditorUrl = url;
            return View(model);
        }

        [GadgetAction(ActionType.Menu, TextResourceKey = "Edit", ResourceType = typeof(EPiServer.Shell.UI.Views.Shared.SharedResources))]
        public ActionResult Configure(Guid gadgetId)
        {
            ViewModel model = Data.LoadViewModel(gadgetId);
            return View("Configure", model.Settings);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult StoreConfiguration(Guid gadgetId, [Bind(Exclude = "gadgetId")] Settings PixlrSettings)
        {
            if (PixlrSettings.EditorHeight==0)
            {
                ModelState.AddModelError("EditorHeight", "Height must be numeric and represents the hight (in pixels) used for the editor.");
            }

            if (PixlrSettings.EditorHeight < 600)
            {
                ModelState.AddModelError("EditorHeight", "The minimal value for height is 600px.");
            }

            if (PixlrSettings.PreviewSize==0)
            {
                ModelState.AddModelError("PreviewSize", "Preview size must be numeric and represents the maximum height/width (in pixels) for the preview.");
            }

            if (PixlrSettings.PreviewSize < 150)
            {
                ModelState.AddModelError("PreviewSize", "The minimal value for preview size is 150px.");
            }

            // If the model isn't valid we return the configuration view again to let the user correct the errors.
            if (!ModelState.IsValid)
            {
                
                return View("Configure", PixlrSettings);
            }

            // Validation successful; persist changes to data store
            Data.SaveSettings(gadgetId, PixlrSettings);

            ViewModel model = GetIndexState(gadgetId);
            return View("Index",model);
        }

        private ViewModel GetIndexState(Guid gadgetId)
        {
            ViewModel model = Data.LoadViewModel(gadgetId);

            // we want to clear SavedFilePath so it's only displayed once
            string savedfilepath = string.Empty;
            if (model.FileInfo != null)
            {
                savedfilepath = model.FileInfo.SavedFilePath;
                model.FileInfo.SavedFilePath = string.Empty;
                Data.SaveFileInfo(gadgetId, model.FileInfo);
                model.FileInfo.SavedFilePath = savedfilepath;
            }

            return model;
        }

        private string Locale
        {
            get
            {
                return CultureInfo.CurrentUICulture.Name;
            }
        }
        private string SiteUrl
        {
            get
            {
                string url = EPiServer.Configuration.Settings.Instance.SiteUrl.ToString();
                return url.Remove(url.LastIndexOf('/'));
            }
        }
        private string SiteName
        {
            get
            {
                return EPiServer.Configuration.Settings.Instance.SiteDisplayName;
            }
        }

        private static string Translate(string key)
        {
            return LanguageManager.Instance.Translate(key);
        }

        #region IGadgetNotify Members
        /// <summary>
        /// Deletes all stored data for the specified gadget id.
        /// </summary>
        /// <param name="gadgetId">The gadget id.</param>
        void IGadgetNotify.OnDeleted(Guid gadgetId)
        {
            Data.ClearGadgetValues(gadgetId);
        }

        #endregion

    }
}
